package org.expeditee.gui;

import org.expeditee.items.Item;
import org.expeditee.items.Line;
import org.expeditee.items.Text;
import org.expeditee.settings.experimental.ExperimentalFeatures;


public class Help {
	
	// contexts
	private static final int background = 0, item = 1, line = 2, text = 3;
	
	// mouse buttons
	private static final int left = 1, middle = 2, right = 4;
	private static final String[] buttons = { "", "L", "M", "LM", "R", "LR", "MR" };
	
	// modifiers
	private static final int control = 1, shift = 2;
	private static final String[] modifiers = { "", "^", "+", "^+" };
	
	// other
	private static final int panning = 1, action = 2, cursor = 4;
	
	// separator between command and description
	private static final String separatorString = ": ";
	// flag for dragging
	private static final String dragString = "d";
	
	private static final String commandPadding = "     ";
	private static final String padding = "                  ";
	
	private static final String command(int mod, int button, boolean drag) {
		String cmd = (drag ? dragString : "") + modifiers[mod] + buttons[button];
		return commandPadding.substring(cmd.length()) + cmd + separatorString;
	}
	
	private static final String command(int mod, int button) {
		return command(mod, button, false);
	}
	
	private static final String left(int context, int mod, int other) {
		switch(context) {
		case background:
    		if((mod & (control | shift)) != 0) return "Go forward";
    		return "Go back";
		default:
			if((other & action) != 0 && (mod & control) == 0) return "Run action";
			return "Follow link";
		}
	}
	
	private static final String middle(int context, int mod, int other) {
		switch(context) {
		case background:
    		if((other & cursor) != 0) return "Place item(s)";
    		return "Create arrow";
		case line:
			if((mod & shift) != 0) return "Extend shape";
		case text:
			if((other & cursor) != 0 && FrameUtils.getCurrentItem() instanceof Text) return "Merge text";
		default:
			if((other & cursor) != 0) return "Place item(s)";
    		if((mod & shift) != 0) return "Create arrow";
			return "Pickup";
		}
	}
	
	private static final String right(int context, int mod, int other) {
		switch(context) {
		case background:
			if((other & cursor) != 0) return "Stamp item(s)";
			return "Create rectangle";
		case line:
			if((mod & shift) != 0) return "Add line";
		default:
			if((other & cursor) != 0) return "Stamp item(s)";
			if((mod & shift) != 0) return "Create rectangle";
			return "Copy";
		}
	}
	
	private static final String left_middle(int context, int mod, int other) {
		return null;
	}
	
	private static final String left_right(int context, int mod, int other) {
		switch(context) {
		default:
			if((other & cursor) == 0) return "Extract attributes";
		case background:
			return "Auto format";
		}
	}
	
	private static final String middle_right(int context, int mod, int other) {
		switch(context) {
		case text:
			if((other & cursor) != 0 && FrameUtils.getCurrentItem() instanceof Text) return "Swap text";
		case background:
			if((other & cursor) == 0) return "Undo";
		default:
			return "Delete";
		}
	}
	
	private static final String drag_left(int context, int mod, int other) {
		switch(context) {
		case background:
    		if((mod & shift) != 0 && (other & cursor) == 0) return "Drag to pan";
		case text:
			if((other & cursor) == 0) return "Select region";
		}
		return null;
	}
	
	private static final String drag_middle(int context, int mod, int other) {
		switch(context) {
		case text:
			if((other & cursor) == 0) return "Cut region";
		}
		return null;
	}
	
	private static final String drag_right(int context, int mod, int other) {
		switch(context) {
		case line:
			return "Extrude shape";
		case text:
			if((other & cursor) == 0) return "Copy region";
		}
		return null;
	}
	
	
	public static void updateStatus() {
		Item current = FrameUtils.getCurrentItem();
		int context;
		if(current != null) {
			if(current instanceof Line) {
				context = line;
			} else if(current instanceof Text) {
				context = text;
			} else {
				context = item;
			}
		} else if(FrameUtils.getCurrentItems() != null) {
			context = item;
		} else {
			context = background;
		}
		int mod = (FrameMouseActions.isControlDown() ? control : 0) | (FrameMouseActions.isShiftDown() ? shift : 0);
		int other = (ExperimentalFeatures.MousePan.get() ? panning : 0) | (current != null && current.hasAction() ? action : 0) | (FreeItems.itemsAttachedToCursor() ? cursor : 0);
		String status = "";
		
		String l =  left(context, mod, other);
		String m =  middle(context, mod, other);
		String r =  right(context, mod, other);
		String lr = left_right(context, mod, other);
		String lm = left_middle(context, mod, other);
		String mr = middle_right(context, mod, other);
		String dl = drag_left(context, mod, other);
		String dm = drag_middle(context, mod, other);
		String dr = drag_right(context, mod, other);
		
		if(l != null)  status += command(mod, left) + l + padding.substring(l.length());
		if(m != null)  status += command(mod, middle) + m + padding.substring(m.length());
		if(r != null)  status += command(mod, right) + r + padding.substring(r.length());
		if(lm != null) status += command(mod, left | middle) + lm + padding.substring(lm.length());
		if(lr != null) status += command(mod, left | right) + lr + padding.substring(lr.length());
		if(mr != null) status += command(mod, middle | right) + mr + padding.substring(mr.length());
		if(dl != null || dm != null || dr != null) status += "\n";
		if(dl != null) status += command(mod, left, true) + dl + padding.substring(dl.length());
		if(dm != null) status += command(mod, middle, true) + dm + padding.substring(dm.length());
		if(dr != null) status += command(mod, right, true) + dr + padding.substring(dr.length());
		
		MessageBay.setStatus(status);
	}

}
